local _, addonTable = ...
local E, L, C = addonTable.E, addonTable.L, addonTable.C

-- Lua
local _G = getfenv(0)

-- Blizz
local C_TradeSkillUI = _G.C_TradeSkillUI

--[[ luacheck: globals
	GameTooltip GetSpellInfo GetSpellRank InCombatLockdown TradeSkillFrame TradeSkillFrame_LoadUI
]]

-- Mine
local function Toast_OnClick(self)
	if self._data.tradeskill_id and not InCombatLockdown() then
		if not TradeSkillFrame then
			TradeSkillFrame_LoadUI()
		end

		if TradeSkillFrame then
			if C_TradeSkillUI.OpenTradeSkill(self._data.tradeskill_id) then
				TradeSkillFrame:SelectRecipe(self._data.recipe_id)
			end
		end
	end
end

local function Toast_OnEnter(self)
	if self._data.recipe_id then
		GameTooltip:SetSpellByID(self._data.recipe_id)
		GameTooltip:Show()
	end
end

local rankTextures = {
	[1] = "|TInterface\\LootFrame\\toast-star:12:12:0:0:32:32:0:21:0:21|t",
	[2] = "|TInterface\\LootFrame\\toast-star-2:12:24:0:0:64:32:0:42:0:21|t",
	[3] = "|TInterface\\LootFrame\\toast-star-3:12:36:0:0:64:32:0:64:0:21|t",
}

local function Toast_SetUp(event, recipeID, recipeLevel)
	local tradeSkillID = C_TradeSkillUI.GetTradeSkillLineForRecipe(recipeID)
	if tradeSkillID then
		local recipeName = GetSpellInfo(recipeID)
		if recipeName then
			local toast = E:GetToast()

			if recipeLevel then
				toast.IconText1:SetText(rankTextures[recipeLevel])
				toast.IconText1BG:Show()
			end

			toast:SetBackground("recipe")
			toast.Title:SetText(recipeLevel and recipeLevel > 1 and L["RECIPE_UPGRADED"] or L["RECIPE_LEARNED"])
			toast.Text:SetText(recipeName)
			toast.Icon:SetTexture(C_TradeSkillUI.GetTradeSkillTexture(tradeSkillID))
			toast.IconBorder:Show()

			toast._data.event = event
			toast._data.recipe_id = recipeID
			toast._data.sound_file = C.db.profile.types.recipe.sfx and 73919 -- SOUNDKIT.UI_PROFESSIONS_NEW_RECIPE_LEARNED_TOAST
			toast._data.tradeskill_id = tradeSkillID

			toast:HookScript("OnClick", Toast_OnClick)
			toast:HookScript("OnEnter", Toast_OnEnter)
			toast:Spawn(C.db.profile.types.recipe.anchor, C.db.profile.types.recipe.dnd)
		end
	end
end

local function NEW_RECIPE_LEARNED(...) -- recipeID, recipeLevel
	Toast_SetUp("NEW_RECIPE_LEARNED", ...)
end

local function Enable()
	if C.db.profile.types.recipe.enabled then
		E:RegisterEvent("NEW_RECIPE_LEARNED", NEW_RECIPE_LEARNED)
	end
end

local function Disable()
	E:UnregisterEvent("NEW_RECIPE_LEARNED", NEW_RECIPE_LEARNED)
end

local function Test()
	-- no rank, Elixir of Minor Defence
	Toast_SetUp("RECIPE_TEST", 7183)

	-- rank 2, Word of Critical Strike
	Toast_SetUp("RECIPE_TEST", 190992, GetSpellRank(190992))
end

E:RegisterOptions("recipe", {
	enabled = true,
	anchor = 1,
	dnd = false,
	sfx = true,
}, {
	name = L["TYPE_RECIPE"],
	get = function(info)
		return C.db.profile.types.recipe[info[#info]]
	end,
	set = function(info, value)
		C.db.profile.types.recipe[info[#info]] = value
	end,
	args = {
		enabled = {
			order = 1,
			type = "toggle",
			name = L["ENABLE"],
			set = function(_, value)
				C.db.profile.types.recipe.enabled = value

				if value then
					Enable()
				else
					Disable()
				end
			end
		},
		dnd = {
			order = 2,
			type = "toggle",
			name = L["DND"],
			desc = L["DND_TOOLTIP"],
		},
		sfx = {
			order = 3,
			type = "toggle",
			name = L["SFX"],
		},
		test = {
			type = "execute",
			order = 99,
			width = "full",
			name = L["TEST"],
			func = Test,
		},
	},
})

E:RegisterSystem("recipe", Enable, Disable, Test)
